@@ -1,6 +1,6 @@
 /*  ipguard.h
  *
- * Copyright (c) 2008 SeaD <sead at deep.perm.ru>
+ * Copyright (c) 2010 SeaD <sead at deep.perm.ru>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -23,6 +23,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
+ *  $Id: ipguard.h,v 1.17 2010/07/12 03:46:36 sead Exp $
+ *
  */
 
 #ifndef _IPGUARD_H
@@ -45,7 +47,7 @@
 #endif
 
 #define NAME            "ipguard"
-#define VERSION         "1.02"
+#define VERSION         "1.04"
 #define AUTHOR          "SeaD"
 #define MAIL            "sead at deep.perm.ru"
 
@@ -53,9 +55,10 @@
 #define LOGNAME         "/var/log/ipguard"
 #define PIDNAME         "/var/run/ipguard"
 #define FAKEMAC         "de:ad:xx:xx:xx:xx"
-#define PCAP_FILTER     "arp"
+#define PCAPFSIZ        128
 
 #define ETHERSTO        0
+#define FAKEREGEN       0
 #define FAKENUM         2
 #define FAKETIME        50      /* milliseconds */
 #define BUFSIZE         10
@@ -70,8 +73,10 @@ char ethers_name[PATH_MAX];
 char log_name[PATH_MAX];
 char pid_name[PATH_MAX];
 char fmac[18];
+char pcapf[PCAPFSIZ];
 char suser[MAXLOGNAME];
 int ethers_update;
+int fake_regen;
 int fake_num;
 int fake_time;
 int buffer_num;
@@ -87,7 +92,8 @@ int debug;
 int verbose;
 
 unsigned int all, good, grat, wgrat, zmac, zip, bad, bmac, bsip, btip,
-    bnew, bgrat, mymac, fake, nzh, nbe, mis;
+    bnew, bgrat, mymac, fake, pfake, nzh, nbe, mis;
+char pfmac[18];
 char s[128+1];
 
 /*  ethers.c
@@ -111,7 +117,6 @@ void stat_dump(void);
 /*  system.c
  */
 void exit_ipguard(int reason);
-void rand_mac(char *mac);
 void sig_init(void);
 void sig_func(int signal);
 void sig_catch(void);
@@ -123,8 +128,9 @@ void pid_creat(void);
 void pid_unlink(void);
 void daemonize(void);
 void set_user(void);
+void mac_rand(char *mac);
+void mac_regen(char *mac);
 char *time_get(void);
-void ethers_rand(char *mac);
 void ethers_stat(void);
 
 #endif  /* _IPGUARD_H */
