@@ -1,6 +1,6 @@
 /*  packet.c
  *
- * Copyright (c) 2008 SeaD <sead at deep.perm.ru>
+ * Copyright (c) 2010 SeaD <sead at deep.perm.ru>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -23,6 +23,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
+ *  $Id: packet.c,v 1.19 2010/07/12 03:46:36 sead Exp $
+ *
  */
 
 #include <sys/types.h>
@@ -53,7 +55,8 @@
 
 #include "ipguard.h"
 
-static struct ether_addr se_addr, te_addr, sh_addr, th_addr, bh_addr, zh_addr, fake_addr;
+static struct ether_addr se_addr, te_addr, sh_addr, th_addr, bh_addr, zh_addr,
+    fake_addr, pfake_addr;
 static struct in_addr sp_addr, tp_addr, zp_addr;
 
 static pcap_t *pcap = NULL;
@@ -63,6 +66,7 @@ static char ifname[IFNAMSIZ];
 static struct pair_t {
     struct ether_addr mac;
     struct in_addr ip;
+    struct in_addr mask;
 } *pairs = NULL;
 static int pair_num = 0;
 
@@ -125,9 +129,19 @@ void buffer_dump2ethers(void) {
     }
 }
 
+uint32_t bit_2mask(int bit) {
+    uint32_t mask = 0;
+    register int n;
+
+    for (n = 0; n < bit; n++) { mask >>= 1; mask |= 0x80000000; }
+
+    return mask;
+}
+
 void pair_init(void) {
     struct libnet_ether_addr *ha = NULL;
-    u_long pa = 0;
+    uint32_t pa = 0, mask;
+    int bit = 0;
 
     if ((pairs = (struct pair_t *) malloc(sizeof(struct pair_t))) == NULL) {
         log_str(ERROR, "malloc():", strerror(errno));
@@ -141,13 +155,16 @@ void pair_init(void) {
         log_str(ERROR, libnet_geterror(ln), "");
         exit(EXIT_FAILURE);
     }
+    mask = htonl(bit_2mask(bit));
     if (!nofirst) {
         memcpy(&pairs->mac, ha, sizeof(struct ether_addr));
         memcpy(&pairs->ip, &pa, sizeof(struct in_addr));
+        memcpy(&pairs->mask, &mask, sizeof(struct in_addr));
         pair_num++;
 
         if (debug > 1)
-            fprintf(stderr, "PAIR ADD FIRST: %s %s\n", ether_ntoa(&pairs->mac), inet_ntoa(pairs->ip));
+            fprintf(stderr, "PAIR ADD FIRST: %s %s %u\n",
+                ether_ntoa(&pairs->mac), inet_ntoa(pairs->ip), bit);
     }
 }
 
@@ -159,7 +176,17 @@ void pair_destroy(void) {
 void pair_add(char *mac, char *ip) {
     struct pair_t *p = NULL;
     struct ether_addr *fmac = NULL;
-    u_long fip;
+    uint32_t fip, mask;
+    char *m;
+    int bit;
+
+    if ((m = strchr(ip, '/'))) {
+        *m++ = '\0'; bit = atoi(m);
+        if (!bit || (bit < 0) || (bit > 32)) {
+            log_str(NOTICE, "Wrong mask", m); return;
+        }
+        if (debug > 2) fprintf(stderr, "NETWORK MASK: %s %u\n", ip, bit);
+    } else bit = 0;
 
     if ((fmac = ether_aton(mac)) == NULL) {
         if (verbose) log_str(WARNING, "Invalid MAC:", mac); return;
@@ -168,6 +195,9 @@ void pair_add(char *mac, char *ip) {
         if (verbose) log_str(WARNING, "Host unknown:", ip); return;
     }
 
+    mask = htonl(bit_2mask(bit));
+    if (mask) fip &= mask;
+
     if ((pairs = realloc(pairs, ++pair_num * sizeof(struct pair_t))) == NULL) {
         log_str(ERROR, "realloc():", strerror(errno));
         exit(EXIT_FAILURE);
@@ -176,24 +206,41 @@ void pair_add(char *mac, char *ip) {
 
     memcpy(&p->mac, fmac, sizeof(struct ether_addr));
     memcpy(&p->ip, &fip, sizeof(struct in_addr));
+    memcpy(&p->mask, &mask, sizeof(struct in_addr));
 
     if (debug > 1)
-        fprintf(stderr, "PAIR ADD: %s %s\n", ether_ntoa(&p->mac), inet_ntoa(p->ip));
+        fprintf(stderr, "PAIR ADD: %s %s %u\n",
+            ether_ntoa(&p->mac), inet_ntoa(p->ip), bit);
 }
 
 void pair_dump(void) {
     struct pair_t *p = pairs;
+    char ntoasw[16];
     register int n;
 
     snprintf(s, 64, "Current MAC-IP table (%d pairs)", pair_num);
     log_str(INFO, s, "");
 
     for (n = 0; n < pair_num; n++, p++) {
-        snprintf(s, 64, "%s %s", ether_ntoa(&p->mac), inet_ntoa(p->ip));
+        strncpy(ntoasw, inet_ntoa(p->mask), 16);
+        snprintf(s, 64, "%s %s %s",
+            ether_ntoa(&p->mac), inet_ntoa(p->ip), ntoasw);
         log_str(INFO, "dump:", s);
     }
 }
 
+void packet_fmacs(int op) {
+    switch (op) {
+        /* init */
+        case 1: mac_rand(fmac); strncpy(pfmac, fmac, 18); break;
+        /* regenerate */
+        case 2: strncpy(pfmac, fmac, 18); mac_regen(fmac); break;
+    }
+
+    memcpy(&fake_addr, ether_aton(fmac), sizeof(struct ether_addr));
+    memcpy(&pfake_addr, ether_aton(pfmac), sizeof(struct ether_addr));
+}
+
 void packet_init(char *iface) {
     struct bpf_program arp_p;
     char ebuf[PCAP_ERRBUF_SIZE] = "\0";
@@ -202,27 +249,27 @@ void packet_init(char *iface) {
     memset(&bh_addr, 0xff, sizeof(struct ether_addr));
     memset(&zh_addr, 0x00, sizeof(struct ether_addr));
     memset(&zp_addr, 0x00, sizeof(struct in_addr));
-    rand_mac(fmac);
-    memcpy(&fake_addr, ether_aton(fmac), sizeof(struct ether_addr));
+    packet_fmacs(1);
 
     /* pcap init */
     if (!(pcap = pcap_open_live(ifname, 100, promisc, 10, ebuf))) {
-        debug++; log_str(ERROR, "pcap_open_live():", ebuf);
+        debug = 2; log_str(ERROR, "pcap_open_live():", ebuf);
         exit(EXIT_FAILURE);
     }
     if (strlen(ebuf)) fprintf(stderr, "warning: %s\n", ebuf);
-    if (pcap_compile(pcap, &arp_p, PCAP_FILTER, 0, -1) == -1) {
-        debug++; log_str(ERROR, "pcap_compile():", "failed");
+    if (pcap_compile(pcap, &arp_p, pcapf, 0, -1) == -1) {
+        snprintf(s, 128, "pcap_compile(%s):", pcapf);
+        debug = 2; log_str(ERROR, s, "wrong expression");
         exit(EXIT_FAILURE);
     }
     if (pcap_setfilter(pcap, &arp_p) == -1) {
-        debug++; log_str(ERROR, "pcap_setfilter():", "failed");
+        debug = 2; log_str(ERROR, "pcap_setfilter():", "failed");
         exit(EXIT_FAILURE);
     }
 
     /* libnet init */
     if (!(ln = libnet_init(LIBNET_LINK_ADV, ifname, ebuf))) {
-        debug++; log_str(ERROR, ebuf, "");
+        debug = 2; log_str(ERROR, ebuf, "");
         exit(EXIT_FAILURE);
     }
 
@@ -230,8 +277,9 @@ void packet_init(char *iface) {
     if (buffer_num) buffer_init();
     ethers_init();
 
-    snprintf(s, 128, "%s %s %s (%d pairs) fake %s",
-        iface, ether_ntoa(&pairs->mac), inet_ntoa(pairs->ip), pair_num, fmac);
+    snprintf(s, 128, "%s %s %s (%d pairs) fake %s%s",
+        iface, ether_ntoa(&pairs->mac), inet_ntoa(pairs->ip), pair_num, fmac,
+        (read_only ? " RO mode" : ""));
     log_str(NOTICE, s, "");
     if (verbose) pair_dump();
 }
@@ -282,35 +330,32 @@ void packet_send(int op, int num) {
 void packet_check(void) {
     struct pair_t *p = NULL;
     struct pair_t *se_ch = NULL, *te_ch = NULL, *sp_ch = NULL, *tp_ch = NULL;
+    uint32_t fsip, ftip, fmask, swip;
     int bsdr = 0;
-    u_long ipsw;
     char ntoasw[16];
     register int n;
 
     /* count mismatch MACs between ether and ARP headers */
     if (memcmp(&se_addr, &sh_addr, sizeof(struct ether_addr))) {
-        if (verbose) {
+        if (debug > 1) {
             strncpy(ntoasw, inet_ntoa(tp_addr), 16);
-            snprintf(s, 128, "%-17s %-15s %-15s", ether_ntoa(&sh_addr), inet_ntoa(sp_addr), ntoasw);
-            log_str(WARNING, "e!=h:", s);
+            fprintf(stderr, "MISMATCH SOURCE: %-17s %-15s %-15s\n", ether_ntoa(&sh_addr), inet_ntoa(sp_addr), ntoasw);
         }
         mis++;
     }
     /* count non-zero target MAC in ARP who-has */
     if (memcmp(&zh_addr, &th_addr, sizeof(struct ether_addr))) {
-        if (verbose) {
+        if (debug > 1) {
             strncpy(ntoasw, inet_ntoa(tp_addr), 16);
-            snprintf(s, 128, "%-17s %-15s %-15s", ether_ntoa(&sh_addr), inet_ntoa(sp_addr), ntoasw);
-            log_str(WARNING, "!z h:", s);
+            fprintf(stderr, "NON-ZERO TARGET: %-17s %-15s %-15s\n", ether_ntoa(&sh_addr), inet_ntoa(sp_addr), ntoasw);
         }
         nzh++;
     }
     /* count non-bcast target ether MAC */
     if (memcmp(&bh_addr, &te_addr, sizeof(struct ether_addr))) {
-        if (verbose) {
+        if (debug > 1) {
             strncpy(ntoasw, inet_ntoa(tp_addr), 16);
-            snprintf(s, 128, "%-17s %-15s %-15s", ether_ntoa(&sh_addr), inet_ntoa(sp_addr), ntoasw);
-            log_str(WARNING, "!b e:", s);
+            fprintf(stderr, "NON-BCAST WHO-HAS: %-17s %-15s %-15s\n", ether_ntoa(&sh_addr), inet_ntoa(sp_addr), ntoasw);
         }
         nbe++;
     }
@@ -325,6 +370,12 @@ void packet_check(void) {
         if (debug > 2) fprintf(stderr, "FAKE MAC PASSED: %s\n", ether_ntoa(&se_addr));
         fake++; return;
     }
+    /* return if sdr MAC is previous fake MAC */
+    if (fake_regen)
+        if (!memcmp(&se_addr, &pfake_addr, sizeof(struct ether_addr))) {
+            if (debug > 2) fprintf(stderr, "PRE FAKE MAC PASSED: %s\n", ether_ntoa(&se_addr));
+            pfake++; return;
+        }
     /* fill weird gratuitous ARP sdr IP 0.0.0.0 (Linux/MacOS/Vista) */
     if (!memcmp(&sp_addr, &zp_addr, sizeof(struct in_addr))) {
         if (debug > 2) fprintf(stderr, "WEIRD GRATUITOUS FILL: %s\n", inet_ntoa(tp_addr));
@@ -334,16 +385,20 @@ void packet_check(void) {
 
     /* check if IP/MAC is listed */
     for (n = 0, p = pairs; n < pair_num; n++, p++) {
+        memcpy(&fsip, &sp_addr, sizeof(struct in_addr));
+        memcpy(&ftip, &tp_addr, sizeof(struct in_addr));
+        memcpy(&fmask, &p->mask, sizeof(struct in_addr));
+        if (fmask) { fsip &= fmask; ftip &= fmask; }
         if (!memcmp(&se_addr, &p->mac, sizeof(struct ether_addr))) {
             if (!se_ch) se_ch = p;
-            if (!memcmp(&sp_addr, &p->ip, sizeof(struct in_addr))) se_ch = sp_ch = p;
+            if (!memcmp(&fsip, &p->ip, sizeof(struct in_addr))) se_ch = sp_ch = p;
         }
         if (!memcmp(&te_addr, &p->mac, sizeof(struct ether_addr))) {
             if (!te_ch) te_ch = p;
-            if (!memcmp(&tp_addr, &p->ip, sizeof(struct in_addr))) te_ch = tp_ch = p;
+            if (!memcmp(&ftip, &p->ip, sizeof(struct in_addr))) te_ch = tp_ch = p;
         }
-        if (!sp_ch && !memcmp(&sp_addr, &p->ip, sizeof(struct in_addr))) sp_ch = p;
-        if (!tp_ch && !memcmp(&tp_addr, &p->ip, sizeof(struct in_addr))) tp_ch = p;
+        if (!sp_ch && !memcmp(&fsip, &p->ip, sizeof(struct in_addr))) sp_ch = p;
+        if (!tp_ch && !memcmp(&ftip, &p->ip, sizeof(struct in_addr))) tp_ch = p;
     }
 
     if (debug > 2) {
@@ -451,9 +506,9 @@ void packet_check(void) {
     memcpy(&se_addr, &fake_addr, sizeof(struct ether_addr));
     memcpy(&th_addr, &sh_addr, sizeof(struct ether_addr));
     memcpy(&sh_addr, &fake_addr, sizeof(struct ether_addr));
-    memcpy(&ipsw, &tp_addr, sizeof(struct in_addr));
+    memcpy(&swip, &tp_addr, sizeof(struct in_addr));
     memcpy(&tp_addr, &sp_addr, sizeof(struct in_addr));
-    memcpy(&sp_addr, &ipsw, sizeof(struct in_addr));
+    memcpy(&sp_addr, &swip, sizeof(struct in_addr));
     packet_send(ARPOP_REPLY, fake_num);
 
     if (bsdr) {
@@ -505,7 +560,7 @@ void packet_check(void) {
     bad++;
     if (debug > 2)
         fprintf(stderr, "STATS:  god %-6d  new %-6d  bmc %-6d  bip %-6d  btg %-6d  all %-6d\n",
-            good+mymac+fake+grat+wgrat+zmac+zip, bnew, bmac, bsip, btip, all);
+            good+mymac+fake+pfake+grat+wgrat+zmac+zip, bnew, bmac, bsip, btip, all);
 }
 
 void packet_handle(char *blah, struct pcap_pkthdr *blah2, u_char *recv) {
@@ -515,6 +570,7 @@ void packet_handle(char *blah, struct pcap_pkthdr *blah2, u_char *recv) {
 
     sig_catch();
     if (ethers_update) ethers_stat();
+    if (fake_regen) packet_fmacs(2);
 
     eth_h = (struct ether_header *) recv;
     if (htons(eth_h->ether_type) != ETHERTYPE_ARP) {
@@ -577,21 +633,21 @@ void stat_dump(void) {
     log_str(INFO, s, "");
     snprintf(s, 64, "Gratuitous MAC-IP/weird %u/%u", grat, wgrat);
     log_str(INFO, s, "");
-    snprintf(s, 64, "Zero MACs/IPs %u/%u", zmac, zip);
+    snprintf(s, 64, "Zero MAC/IP %u/%u", zmac, zip);
     log_str(INFO, s, "");
     snprintf(s, 64, "New MAC-IP pairs %u", bnew);
     log_str(INFO, s, "");
     snprintf(s, 64, "Bad MAC-IP pairs %u", bad);
     log_str(INFO, s, "");
-    snprintf(s, 64, "Bad MACs/IPs %u/%u", bmac, bsip);
+    snprintf(s, 64, "Bad MAC/IP %u/%u", bmac, bsip);
     log_str(INFO, s, "");
     if (duplex) {
-        snprintf(s, 64, "Bad target IPs %u", btip);
+        snprintf(s, 64, "Bad target IP %u", btip);
         log_str(INFO, s, "");
     }
     snprintf(s, 64, "Bad gratuitous MAC-IP %u", bgrat);
     log_str(INFO, s, "");
-    snprintf(s, 64, "My MACs/Fake MACs %u/%u", mymac, fake);
+    snprintf(s, 64, "My/Fake/PreFake MAC %u/%u/%u", mymac, fake, pfake);
     log_str(INFO, s, "");
     snprintf(s, 64, "Mismatch sender Ether/ARP MAC %u", mis);
     log_str(INFO, s, "");
@@ -608,3 +664,4 @@ void stat_dump(void) {
     snprintf(s, 64, "Received/Dropped packets %u/%u", ps.ps_recv, ps.ps_drop);
     log_str(INFO, s, "");
 }
+
