@@ -1,6 +1,6 @@
 /*  system.c
  *
- * Copyright (c) 2008 SeaD <sead at deep.perm.ru>
+ * Copyright (c) 2010 SeaD <sead at deep.perm.ru>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -23,6 +23,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
+ *  $Id: system.c,v 1.15 2010/07/12 03:46:36 sead Exp $
+ *
  */
 
 #include <pwd.h>
@@ -30,6 +32,7 @@
 #include <paths.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 
 #ifdef __linux__
 # include <time.h>
@@ -108,6 +111,7 @@ void log_str(int pri, char *ent, char *err) {
         case INFO: strncpy(p, "info", 10); break;
         default: strncpy(p, "unknown", 10); break;
     }
+    if (debug > 1) fprintf(stderr, "%s ", time_get());
     if (debug) fprintf(stderr, "%s %s\n", ent, err);
     fprintf(log_fp, "%s %s %s %s\n", time_get(), p, ent, err);
     fflush(log_fp);
@@ -243,7 +247,7 @@ void set_user(void) {
     if (verbose) log_str(NOTICE, "Running as uid/gid:", s);
 }
 
-void rand_mac(char *mac) {
+void mac_rand(char *mac) {
     char c[] = "  ";
     register int n;
 
@@ -251,6 +255,26 @@ void rand_mac(char *mac) {
         if (mac[n] == 'x') { snprintf(c, 2, "%1x", rand() % 16); mac[n] = c[0]; }
 }
 
+void mac_regen(char *mac) {
+    static time_t count = 0;
+    time_t t;
+    char c[] = "  ";
+    register int n;
+
+    if ((t = time(&t)) < count) return;
+    count = t + fake_regen;
+
+    if (debug > 1) fprintf(stderr, "FAKE REGEN: pre %-17s ", mac);
+
+    for (n = 0; n < 18; n++)
+        if ((mac[n] >= '0' && mac[n] <= '9') ||
+            (mac[n] >= 'A' && mac[n] <= 'F') ||
+            (mac[n] >= 'a' && mac[n] <= 'f'))
+            { snprintf(c, 2, "%1x", rand() % 16); mac[n] = c[0]; }
+
+    if (debug > 1) fprintf(stderr, "new %-17s\n", mac);
+}
+
 char *time_get(void) {
     static char cur_time[16];
     time_t t;
@@ -258,7 +282,7 @@ char *time_get(void) {
     t = time(&t);
     strncpy(cur_time, (char *) ctime(&t) + 4, 15);
     cur_time[15] = '\0';
-    return (cur_time);
+    return cur_time;
 }    
 
 void ethers_stat(void) {
@@ -269,18 +293,14 @@ void ethers_stat(void) {
     if ((t = time(&t)) < count) return;
     count = t + ethers_update;
 
-    if (!ethers_mtime) {
-        if (stat(ethers_name, &es) == -1) {
-            snprintf(s, 128, "stat(%s):", ethers_name);
-            log_str(WARNING, s, strerror(errno));
-        }
-        ethers_mtime = es.st_mtime;
-    }
-
     if (stat(ethers_name, &es) == -1) {
         snprintf(s, 128, "stat(%s):", ethers_name);
         log_str(WARNING, s, strerror(errno));
+        return;
     }
+
+    if (!ethers_mtime) ethers_mtime = es.st_mtime;
+
     if (ethers_mtime < es.st_mtime) {
         ethers_reinit(); ethers_mtime = es.st_mtime;
     }
